@@ -52,8 +52,11 @@ public:
     int mDepth;
     QString mDir, mName, mInternalName, mDesc;
     QStringList mInherits;
-    QList<KIconThemeDir *> mDirs;
+    QStringList mExtensions;
+    QVector<KIconThemeDir *> mDirs;
+    bool followsColorScheme : 1;
 };
+Q_GLOBAL_STATIC(QString, _themeOverride)
 Q_GLOBAL_STATIC(QString, _theme)
 Q_GLOBAL_STATIC(QStringList, _theme_list)
 
@@ -71,9 +74,9 @@ public:
     }
     QString iconPath(const QString &name) const;
     QStringList iconList() const;
-    QString dir() const
+    QString constructFileName(const QString &file) const
     {
-        return mBaseDirThemeDir;
+        return mBaseDir + mThemeDir + '/' + file;
     }
 
     KIconLoader::Context context() const
@@ -108,7 +111,8 @@ private:
     int mSize, mMinSize, mMaxSize;
     int mThreshold;
 
-    QString mBaseDirThemeDir;
+    const QString mBaseDir;
+    const QString mThemeDir;
 };
 
 
@@ -116,7 +120,6 @@ private:
 KIconTheme::KIconTheme(const QString &name, const QString &appName, const QString &basePathHint)
     : d(new KIconThemePrivate)
 {
-
     d->mInternalName = name;
 
     QStringList themeDirs;
@@ -130,9 +133,9 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
             (name == defaultThemeName() || name == QLatin1String("hicolor") || name == QLatin1String("locolor"))) {
         const QStringList icnlibs = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
         for (QStringList::ConstIterator it = icnlibs.constBegin(); it != icnlibs.constEnd(); ++it) {
-            const QString cDir = *it + '/' + appName + "/icons/" + name;
+            const QString cDir = *it + '/' + appName + "/icons/" + name + '/';
             if (QFile::exists(cDir)) {
-                themeDirs += cDir + '/';
+                themeDirs += cDir;
             }
         }
 
@@ -189,15 +192,17 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     d->mInherits = cfg.readEntry("Inherits", QStringList());
     if (name != defaultThemeName()) {
         for (QStringList::Iterator it = d->mInherits.begin(); it != d->mInherits.end(); ++it) {
-            if (*it == QLatin1String("default") || *it == QLatin1String("hicolor")) {
+            if (*it == QLatin1String("default")) {
                 *it = defaultThemeName();
             }
         }
     }
 
     d->hidden = cfg.readEntry("Hidden", false);
+    d->followsColorScheme = cfg.readEntry("FollowsColorScheme", false);
     d->example = cfg.readPathEntry("Example", QString());
     d->screenshot = cfg.readPathEntry("ScreenShot", QString());
+    d->mExtensions = cfg.readEntry("KDE-Extensions", QStringList{ ".png", ".svgz", ".svg", ".xpm" });
 
     const QStringList dirs = cfg.readPathEntry("Directories", QStringList());
     for (QStringList::ConstIterator it = dirs.begin(); it != dirs.end(); ++it) {
@@ -228,13 +233,7 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     for (int i = 0; i < groups.size(); ++i) {
         const QString group = groups.at(i);
         d->mDefSize[i] = cg.readEntry(group + "Default", defDefSizes[i]);
-        const QList<int> lst = cg.readEntry(group + "Sizes", QList<int>());
-        QList<int> exp;
-        QList<int>::ConstIterator it2;
-        for (it2 = lst.begin(); it2 != lst.end(); ++it2) {
-                exp += *it2;
-        }
-        d->mSizes[i] = exp;
+        d->mSizes[i] = cg.readEntry(group + "Sizes", QList<int>());
     }
 }
 
@@ -305,10 +304,9 @@ int KIconTheme::defaultSize(KIconLoader::Group group) const
 
 QList<int> KIconTheme::querySizes(KIconLoader::Group group) const
 {
-    QList<int> empty;
     if ((group < 0) || (group >= KIconLoader::LastGroup)) {
         qWarning() << "Illegal icon group: " << group;
-        return empty;
+        return QList<int>();
     }
     return d->mSizes[group];
 }
@@ -400,6 +398,21 @@ bool KIconTheme::hasContext(KIconLoader::Context context) const
     return false;
 }
 
+QString KIconTheme::iconPathByName(const QString &iconName, int size, KIconLoader::MatchType match) const
+{
+    foreach(const QString &current, d->mExtensions) {
+        const QString path = iconPath(iconName + current, size, match);
+        if (!path.isEmpty())
+            return path;
+    }
+    return QString();
+}
+
+bool KIconTheme::followsColorScheme() const
+{
+    return d->followsColorScheme;
+}
+
 QString KIconTheme::iconPath(const QString &name, int size, KIconLoader::MatchType match) const
 {
     QString path;
@@ -482,26 +495,42 @@ QString KIconTheme::iconPath(const QString &name, int size, KIconLoader::MatchTy
 // static
 QString KIconTheme::current()
 {
-    // Static pointer because of unloading problems wrt DSO's.
+    // Static pointers because of unloading problems wrt DSO's.
+    if (_themeOverride && !_themeOverride->isEmpty()) {
+        *_theme() = *_themeOverride();
+    }
     if (!_theme()->isEmpty()) {
         return *_theme();
     }
 
-    KConfigGroup cg(KSharedConfig::openConfig(), "Icons");
-    *_theme() = cg.readEntry("Theme", defaultThemeName());
-    if (*_theme() == QLatin1String("hicolor")) {
-        *_theme() = defaultThemeName();
+    QString theme;
+    // Check application specific config for a theme setting.
+    KConfigGroup app_cg(KSharedConfig::openConfig(QString(), KConfig::NoGlobals), "Icons");
+    theme = app_cg.readEntry("Theme", QString());
+    if (theme.isEmpty() || theme == QLatin1String("hicolor")) {
+        // No theme, try to use Qt's. A Platform plugin might have set
+        // a good theme there.
+        theme = QIcon::themeName();
     }
-    /*    if (_theme->isEmpty())
-        {
-            if (QPixmap::defaultDepth() > 8)
-                *_theme = defaultThemeName();
-            else
-                *_theme = QLatin1String("locolor");
-        }*/
+    if (theme.isEmpty() || theme == QLatin1String("hicolor")) {
+        // Still no theme, try config with kdeglobals.
+        KConfigGroup cg(KSharedConfig::openConfig(), "Icons");
+        theme = cg.readEntry("Theme", QString());
+    }
+    if (theme.isEmpty() || theme == QLatin1String("hicolor")) {
+        // Still no good theme, use default.
+        theme = defaultThemeName();
+    }
+    *_theme() = theme;
     return *_theme();
 }
 
+void KIconTheme::forceThemeForTests(const QString &themeName)
+{
+    *_themeOverride() = themeName;
+    _theme()->clear(); // ::current sets this again based on conditions
+}
+
 // static
 QStringList KIconTheme::list()
 {
@@ -524,10 +553,10 @@ QStringList KIconTheme::list()
 
     Q_FOREACH (const QString &it, icnlibs) {
         QDir dir(it);
-        const QStringList lst = dir.entryList(QDir::Dirs);
+        const QStringList lst = dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
         QStringList::ConstIterator it2;
         for (it2 = lst.begin(); it2 != lst.end(); ++it2) {
-            if ((*it2 == QLatin1String(".")) || (*it2 == QLatin1String("..")) || (*it2).startsWith(QLatin1String("default."))) {
+            if ((*it2).startsWith(QLatin1String("default."))) {
                 continue;
             }
             if (!QFile::exists(it + QLatin1Char('/') + *it2 + QLatin1String("/index.desktop")) &&
@@ -557,7 +586,7 @@ void KIconTheme::reconfigure()
 // static
 QString KIconTheme::defaultThemeName()
 {
-    return QStringLiteral("oxygen");
+    return QStringLiteral("hicolor");
 }
 
 void KIconTheme::assignIconsToContextMenu(ContextMenus type,
@@ -603,7 +632,8 @@ KIconThemeDir::KIconThemeDir(const QString &basedir, const QString &themedir, co
     , mMinSize(1)    // just set the variables to something
     , mMaxSize(50)   // meaningful in case someone calls minSize or maxSize
     , mThreshold(2)
-    , mBaseDirThemeDir(basedir + themedir)
+    , mBaseDir(basedir)
+    , mThemeDir(themedir)
 {
     if (mSize == 0) {
         return;
@@ -639,7 +669,7 @@ KIconThemeDir::KIconThemeDir(const QString &basedir, const QString &themedir, co
     } else if (tmp.isEmpty()) {
         // do nothing. key not required
     } else {
-        qWarning() << "Invalid Context=" << tmp << "line for icon theme: " << dir();
+        qWarning() << "Invalid Context=" << tmp << "line for icon theme: " << constructFileName(QString());
         return;
     }
     tmp = config.readEntry(QStringLiteral("Type"), QStringLiteral("Threshold"));
@@ -650,7 +680,7 @@ KIconThemeDir::KIconThemeDir(const QString &basedir, const QString &themedir, co
     } else if (tmp == QLatin1String("Threshold")) {
         mType = KIconLoader::Threshold;
     } else {
-        qWarning() << "Invalid Type=" << tmp << "line for icon theme: " << dir();
+        qWarning() << "Invalid Type=" << tmp << "line for icon theme: " << constructFileName(QString());
         return;
     }
     if (mType == KIconLoader::Scalable) {
@@ -668,8 +698,7 @@ QString KIconThemeDir::iconPath(const QString &name) const
         return QString();
     }
 
-    QString file = dir() + '/' + name;
-
+    const QString file = constructFileName(name);
     if (QFile::exists(file)) {
         return KLocalizedString::localizedFilePath(file);
     }
@@ -679,15 +708,16 @@ QString KIconThemeDir::iconPath(const QString &name) const
 
 QStringList KIconThemeDir::iconList() const
 {
-    const QDir icondir = dir();
+    const QDir icondir = constructFileName(QString());
 
     const QStringList formats = QStringList() << QStringLiteral("*.png") << QStringLiteral("*.svg") << QStringLiteral("*.svgz") << QStringLiteral("*.xpm");
     const QStringList lst = icondir.entryList(formats, QDir::Files);
 
     QStringList result;
+    result.reserve(lst.size());
     QStringList::ConstIterator it;
-    for (it = lst.begin(); it != lst.end(); ++it) {
-        result += dir() + '/' + *it;
+    foreach(const QString &file, lst) {
+        result += constructFileName(file);
     }
     return result;
 }
